from multiprocessing import Lock, Event, Pipe

class Channel(object):

    def __init__(self):
        self._writelock = Lock()
        self._readlock = Lock()
        self._new_data = Event()
        self._recv_data = Event()
        self._conn1, self._conn2 = Pipe(False)

    def put(self, msg):
        with self._writelock:
            self._conn2.send(msg)
            self._new_data.set()
            self._recv_data.wait()
            self._recv_data.clear()

    def get(self):
        with self._readlock:
            self._new_data.wait()
            msg = self._conn1.recv()
            self._new_data.clear()
            self._recv_data.set()
        return msg


            
## -------------
    
def proc2(channel, stdout_lock):
    from sys import stdout    

    for i in range(1000):

        msg = channel.get()
        
        with stdout_lock:
            print "Process 2 received '%s'\n" % msg,
            stdout.flush()

def proc1(channel, stdout_lock):
    from sys import stdout

    for i in range(1000):

        with stdout_lock:
            print "Process 1 preparing to send 'message %d'\n" % i,
            stdout.flush()

        msg = channel.put(("message %d" % i,))

        with stdout_lock:
            print "Process 1 finished sending 'message %d'\n" % i,
            stdout.flush()
            
if __name__ == "__main__":

    from multiprocessing import Process
                  
    channel = Channel()
    stdout_lock = Lock()
    p2 = Process(target=proc2, args=(channel,stdout_lock))
    p2.start()
    proc1(channel,stdout_lock)
    p2.join()


    

